#include <stdio.h>
#include <Windows.h>
#include <Tlhelp32.h>
#include "bass.h"

#pragma comment(lib, "bass.lib")

#define MUSIC_DIR L"music"
#define MEMORY_OFFSET 0x481390

typedef struct
{
    DWORD id;
    char name[32];
} track_entry;

track_entry track_list[] =
{
    {1000, "TIM"},          {1001, "Unplugged"},        {1007, "Steel Drums"},
    {1002, "New Age"},      {1003, "Hay Seed"},         {1004, "Progressive Rock"},
    {1005, "Salsa"},        {1006, "Techno Rave"},      {1013, "1959 Prom"},
    {1011, "Bongo Bango"},  {1021, "Ragtime"},          {1014, "Hip Hop"},
    {1012, "Keep Tryin'"},  {1017, "Detective Theme"},  {1015, "Dreams"},
    {1016, "Tuna Loaf"},    {1018, "60's Rock"},        {1019, "Pictures"},
    {1020, "Huey Dewey"}
};

DWORD get_pid_by_name(LPCTSTR name)
{
    PROCESSENTRY32 pe32;
    DWORD pid = 0;
    HANDLE ss;
    

    ss = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS | TH32CS_SNAPMODULE32, 0);
    if(ss == INVALID_HANDLE_VALUE)
        return 0;
    
    pe32.dwSize = sizeof(PROCESSENTRY32);

    if(!Process32First(ss, &pe32))
    {
        CloseHandle(ss);
        return 0;
    }
    
    do
    {
        if(!lstrcmp(name, pe32.szExeFile))
        {
            pid = pe32.th32ProcessID;
            break;
        }
    } while(Process32Next(ss, &pe32));
    
    CloseHandle(ss);
    
    return pid;
}

const char * get_track_name_by_id(DWORD id)
{
    int i;

    for(i = 0; i < _countof(track_list); i++)
    {
        if(id == track_list[i].id)
            return track_list[i].name;
    }

    return "unknown";
}

void play_track(LPCTSTR current_dir, DWORD id)
{
    static HSTREAM sm = 0;
    TCHAR file_path[MAX_PATH];

    wsprintf(file_path, L"%s\\%s\\%d.mp3", current_dir, MUSIC_DIR, id);

    if(sm != 0)
        BASS_StreamFree(sm);

    sm = BASS_StreamCreateFile(FALSE, file_path, 0, 0, BASS_UNICODE);
    BASS_ChannelFlags(sm, BASS_SAMPLE_LOOP, BASS_SAMPLE_LOOP);
    BASS_ChannelPlay(sm, FALSE);
}

int main()
{ 
    HANDLE pr;
    DWORD pid, track_id = 0, last_track = 0;
    TCHAR current_dir[MAX_PATH] = {0};

    if(!BASS_Init(-1, 44100, 0, NULL, NULL))
    {
        printf("Failed to init BASS library\n");
        return -1;
    }

    GetCurrentDirectory(_countof(current_dir), current_dir);

    pid = get_pid_by_name(TEXT("TIMWIN.EXE"));
    if(pid == 0)
    {
        printf("Can't find TIMWIN.EXE process\n");
        BASS_Free();

        return -1;
    }


    pr = OpenProcess(PROCESS_VM_READ, FALSE, pid);
    if(pr == NULL)
    {
        printf("Failed to open process (pid=%d)\n", pid);
        BASS_Free();

        return -1;
    }


    while(TRUE)
    {
        if(!ReadProcessMemory(pr, (LPCVOID) MEMORY_OFFSET, &track_id, sizeof(DWORD), NULL))
        {
            printf("Can't read process memory\n");
            break;
        }

        printf("Current track - %08X - %-32s\r", track_id, get_track_name_by_id(track_id));

        if(track_id != last_track)
        {
            play_track(current_dir, track_id);
            last_track = track_id;
        }

        Sleep(100);
    }

    CloseHandle(pr);
    BASS_Free();


    return 0;
}
